package com.app.security

import grails.gorm.DetachedCriteria
import groovy.transform.ToString

import org.codehaus.groovy.util.HashCodeHelper
import grails.compiler.GrailsCompileStatic

@SuppressWarnings(['FactoryMethodName', 'Instanceof'])
@GrailsCompileStatic
@ToString(cache=true, includeNames=true, includePackage=false)
class LoginUserRole implements Serializable {

	private static final long serialVersionUID = 1

	LoginUser user
	LoginRole role

	@Override
	boolean equals(other) {
		if (other instanceof LoginUserRole) {
			other.userId == user?.id && other.roleId == role?.id
		}
	}

	@Override
	int hashCode() {
		int hashCode = HashCodeHelper.initHash()
		if (user) {
			hashCode = HashCodeHelper.updateHash(hashCode, user.id)
		}
		if (role) {
			hashCode = HashCodeHelper.updateHash(hashCode, role.id)
		}
		hashCode
	}

	static LoginUserRole get(long userId, long roleId) {
		criteriaFor(userId, roleId).get()
	}

	static boolean exists(long userId, long roleId) {
		criteriaFor(userId, roleId).count()
	}

	private static DetachedCriteria criteriaFor(long userId, long roleId) {
		LoginUserRole.where {
			user == LoginUser.load(userId) &&
					role == Role.load(roleId)
		}
	}

	static LoginUserRole create(Login user, LoginRole role, boolean flush = false) {
		def instance = new LoginUserRole(user: user, role: role)
		instance.save(flush: flush)
		instance
	}

	static boolean remove(Login u, LoginRole r) {
		if (u != null && r != null) {
			LoginUserRole.where { user == u && role == r }.deleteAll()
		}
	}

	static int removeAll(Login u) {
		u == null ? 0 : LoginUserRole.where { user == u }.deleteAll() as int
	}

	static int removeAll(LoginRole r) {
		r == null ? 0 : LoginUserRole.where { role == r }.deleteAll() as int
	}

	static constraints = {
		role validator: { LoginRole r, LoginUserRole ur ->
			if (ur.user?.id) {
				LoginUserRole.withNewSession {
					if (LoginUserRole.exists(ur.user.id, r.id)) {
						return ['userRole.exists']
					}
				}
			}
		}
	}

	static mapping = {
		id composite: ['user', 'role']
		version false
	}
}