package com.app.security

import grails.gorm.transactions.Transactional
import com.app.security.auth.LoginDataService
import com.app.security.auth.LoginRoleDataService
import com.app.security.auth.LoginUserDataService
import com.app.security.auth.LoginUserRoleDataService

import static org.springframework.security.acls.domain.BasePermission.ADMINISTRATION
import static org.springframework.security.acls.domain.BasePermission.DELETE
import static org.springframework.security.acls.domain.BasePermission.READ
import static org.springframework.security.acls.domain.BasePermission.WRITE
import org.springframework.security.acls.model.ObjectIdentityGenerator
import org.springframework.security.acls.model.ObjectIdentityRetrievalStrategy
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken
import org.springframework.security.core.authority.AuthorityUtils
import org.springframework.security.core.context.SecurityContextHolder as SCH

import grails.compiler.GrailsCompileStatic
import grails.plugin.springsecurity.acl.AclService
import grails.plugin.springsecurity.acl.AclUtilService
import com.app.security.LoginRole
import com.app.security.LoginUser

@GrailsCompileStatic
class DataSourcePopulatorService {

	LoginUserDataService userDataService
	LoginUserRoleDataService userRoleDataService
	LoginRoleDataService roleDataService
	LoginDataService contactDataService

	private static final String[] firstNames = [
		'Bob', 'Mary', 'James', 'Jane', 'Kristy', 'Kirsty', 'Kate', 'Jeni', 'Angela', 'Melanie', 'Kent',
		'William', 'Geoff', 'Jeff', 'Adrian', 'Amanda', 'Lisa', 'Elizabeth', 'Prue', 'Richard', 'Darin',
		'Phillip', 'Michael', 'Belinda', 'Samantha', 'Brian', 'Greg', 'Matthew'
	]

	private static final String[] lastNames = [
		'Smith', 'Williams', 'Jackson', 'Rictor', 'Nelson', 'Fitzgerald', 'McAlpine', 'Sutherland', 'Abbott',
		'Hall', 'Edwards', 'Gates', 'Black', 'Brown', 'Gray', 'Marwell', 'Booch', 'Johnson', 'McTaggart',
		'Parklin', 'Findlay', 'Robinson', 'Giugni', 'Lang', 'Chi', 'Carmichael'
	]

	private static final int createEntities = 50

	AclService aclService
	AclUtilService aclUtilService
	ObjectIdentityRetrievalStrategy objectIdentityRetrievalStrategy

	@Transactional
	void populate() {

		SCH.context.authentication = new UsernamePasswordAuthenticationToken('rod', 'koala',
				AuthorityUtils.createAuthorityList('ROLE_IGNORED'))

		LoginRole userRole = roleDataService.save('ROLE_USER')
		LoginRole supervisorRole = roleDataService.save('ROLE_SUPERVISOR')

		createUser 'rod',    'koala',  true,  userRole, supervisorRole
		createUser 'dianne', 'emu',    true,  userRole
		createUser 'scott',  'wombat', true,  userRole
		createUser 'peter',  'opal',   false, userRole
		createUser 'bill',   'wombat', true,  userRole
		createUser 'bob',    'wombat', true,  userRole
		createUser 'jane',   'wombat', true,  userRole

		def contacts = [['John Smith',       'john@somewhere.com'],
		                ['Michael Citizen',  'michael@xyz.com'],
		                ['Joe Bloggs',       'joe@demo.com'],
		                ['Karen Sutherland', 'karen@sutherland.com'],
		                ['Mitchell Howard',  'mitchell@abcdef.com'],
		                ['Rose Costas',      'rose@xyz.com'],
		                ['Amanda Smith',     'amanda@abcdef.com'],
		                ['Cindy Smith',      'cindy@smith.com'],
		                ['Jonathan Citizen', 'jonathan@xyz.com']
		].collect { nameAndEmail ->
			String name = (String)nameAndEmail[0]
			String email = (String)nameAndEmail[1]
			contactDataService.save(name, email)
		}

		Random random = new Random()

		for (int i = 10; i < createEntities; i++) {
			String firstName = firstNames[random.nextInt(firstNames.size())]
			String lastName = lastNames[random.nextInt(lastNames.size())]
			contacts << new Login(name:"$firstName $lastName", email: "${firstName}@${lastName.toLowerCase()}.com").save()
		}

		
		ObjectIdentityGenerator oig = (ObjectIdentityGenerator)objectIdentityRetrievalStrategy
		for (long i = 1; i < createEntities; i++) {
			aclService.createAcl oig.createObjectIdentity(i, Login.name)
		}

		// Now grant some permissions
		aclUtilService.addPermission contacts[0], 'rod',    ADMINISTRATION
		aclUtilService.addPermission contacts[1], 'rod',    READ
		aclUtilService.addPermission contacts[2], 'rod',    READ
		aclUtilService.addPermission contacts[2], 'rod',    WRITE
		aclUtilService.addPermission contacts[2], 'rod',    DELETE
		aclUtilService.addPermission contacts[3], 'rod',    ADMINISTRATION
		aclUtilService.addPermission contacts[3], 'dianne', ADMINISTRATION
		aclUtilService.addPermission contacts[3], 'scott',  READ
		aclUtilService.addPermission contacts[4], 'dianne', ADMINISTRATION
		aclUtilService.addPermission contacts[4], 'dianne', READ
		aclUtilService.addPermission contacts[5], 'dianne', READ
		aclUtilService.addPermission contacts[5], 'dianne', WRITE
		aclUtilService.addPermission contacts[5], 'dianne', DELETE
		aclUtilService.addPermission contacts[5], 'scott',  READ
		aclUtilService.addPermission contacts[6], 'scott',  ADMINISTRATION
		aclUtilService.addPermission contacts[7], 'dianne', ADMINISTRATION
		aclUtilService.addPermission contacts[7], 'dianne', READ
		aclUtilService.addPermission contacts[7], 'scott',  READ
		aclUtilService.addPermission contacts[8], 'scott',  ADMINISTRATION
		aclUtilService.addPermission contacts[8], 'scott',  READ
		aclUtilService.addPermission contacts[8], 'scott',  WRITE
		aclUtilService.addPermission contacts[8], 'scott',  DELETE

		
		aclUtilService.changeOwner contacts[4], 'dianne'
		aclUtilService.changeOwner contacts[5], 'dianne'
		aclUtilService.changeOwner contacts[6], 'scott'
		aclUtilService.changeOwner contacts[7], 'dianne'
		aclUtilService.changeOwner contacts[8], 'scott'

		def users = ['bill', 'bob', 'jane'] // don't want to mess around with consistent sample data
		def permissions = [ADMINISTRATION, READ, DELETE]

		for (int i = 10; i < createEntities; i++) {
			aclUtilService.addPermission contacts[i - 1], users[random.nextInt(users.size())],
					permissions[random.nextInt(permissions.size())]

			aclUtilService.addPermission contacts[i - 1], users[random.nextInt(users.size())],
					permissions[random.nextInt(permissions.size())]
		}

		
		SCH.clearContext()
	}

	@Transactional
	private void createUser(String username, String password, boolean enabled, LoginRole... roles) {
		LoginUser user = userDataService.save(username, password, enabled)
		for (LoginRole role in roles) {
			userRoleDataService.save(user, role)
		}
	}
}
