package com.app.security

import static org.springframework.security.acls.domain.BasePermission.ADMINISTRATION
import grails.testing.mixin.integration.Integration
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.security.acls.domain.PrincipalSid
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken
import org.springframework.security.core.context.SecurityContextHolder
import grails.transaction.Rollback
import spock.lang.Specification

@Integration
@Rollback
class LoginServiceSpec extends Specification {

	private static final Map<String, String> USERNAMES_TO_PASSWORDS = [
			  rod: 'koala', dianne: 'emu', scott: 'wombat', peter: 'opal']

	private @Autowired LoginService loginService

	def cleanup() {
		SecurityContextHolder.clearContext()
	}

	def 'dianne'() {
		setup:
		makeActiveUser 'dianne' // has ROLE_USER

		when:
		List<Login> contacts = loginService.findAll()

		then:
		4 == contacts.size()

		assertContainsContact 4, contacts
		assertContainsContact 5, contacts
		assertContainsContact 6, contacts
		assertContainsContact 8, contacts

		assertDoestNotContainContact 1, contacts
		assertDoestNotContainContact 2, contacts
		assertDoestNotContainContact 3, contacts
	}

	def 'rod'() {
		setup:
		makeActiveUser 'rod' // has ROLE_SUPERVISOR

		when:
		List<Login> contacts = loginService.findAll()

		then:
		4 == contacts.size()

		assertContainsContact 1, contacts
		assertContainsContact 2, contacts
		assertContainsContact 3, contacts
		assertContainsContact 4, contacts

		assertDoestNotContainContact 5, contacts

		Login c1 = loginService.findById(4)
		loginService.deletePermission c1, new PrincipalSid('bob'), ADMINISTRATION
		loginService.addPermission    c1, new PrincipalSid('bob'), ADMINISTRATION
	}

	def 'scott'() {
		setup:
		makeActiveUser 'scott' 

		when:
		List<Login> contacts = loginService.findAll();

		then:
		5 == contacts.size()

		assertContainsContact 4, contacts
		assertContainsContact 6, contacts
		assertContainsContact 7, contacts
		assertContainsContact 8, contacts
		assertContainsContact 9, contacts

		assertDoestNotContainContact 1, contacts
	}

	private void assertContainsContact(long id, List<Login> contacts) {

		if (contacts.any { Login contact -> contact.id == id }) {
			return
		}

		assert false, "List of contacts should have contained: $id"
	}

	private void assertDoestNotContainContact(long id, List<Login> contacts) {
		if (contacts.any { Login contact -> contact.id == id }) {
			assert false, "List of contact should NOT (but did) contain: $id"
		}
	}

	
	private Login getContact(String id) {
		loginService.findAll().find { it.id == id }
	}

	private void makeActiveUser(String username) {
		SecurityContextHolder.context.authentication = new UsernamePasswordAuthenticationToken(
				  username, USERNAMES_TO_PASSWORDS[username] ?: '')
	}
}
