package com.app.security

import geb.spock.GebReportingSpec
import grails.testing.mixin.integration.Integration
import grails.transaction.Rollback
import com.app.security.pages.AddPage
import com.app.security.pages.ContactsPage
import com.app.security.pages.DeleteConfirmPage
import com.app.security.pages.HomePage
import com.app.security.pages.LoginPage
import spock.lang.Stepwise


@Integration
@Rollback
@Stepwise
class LoginSpec extends GebReportingSpec {
	def 'access home page with unauthenticated user success'() {
		when: 'Unauthenticated user accesses the Home Page'
		to HomePage
		then: 'The page is displayed'
		at HomePage
	}

	def 'access manage page with unauthenticated user sends to login page'() {
		when: 'Unauthenticated user accesses the Manage Page'
		manage.click(LoginPage)
		then: 'The login page is displayed'
		at LoginPage
	}

	def 'authenticated user is sent to original page'() {
		when: 'user authenticates'
		login()
		then: 'The manage page is displayed'
		at ContactsPage
	}

	def 'add login link works'() {
		when: 'user clicks add link'
		addContact.click(AddPage)
		then: 'The add page is displayed'
		at AddPage
	}

	def 'add login'() {
		when: 'add a contact'
		addContact()
		then: 'The add page is displayed'
		at ContactsPage
		and: 'The new contact is displayed'
		contacts.find { it.email == 'rob@example.com' }?.name ==  'Rob Winch'
	}

	def 'delete login'() {
		when: 'delete a login'
		contacts.find { it.email == 'rob@example.com' }.delete()
		then: 'Delete confirmation displayed'
		at DeleteConfirmPage
		when: 'View Manage Page'
		manage.click()
		then: 'New login has been removed'
		!contacts.find { it.email == 'rob@example.com' }
	}

	def 'authenticated user logs out'() {
		when: 'user logs out'
		logout.click()
		then: 'the default logout success page is displayed'
		at HomePage
		when: 'Unauthenticated user accesses the Manage Page'
		via ContactsPage
		then: 'The login page is displayed'
		at LoginPage
	}
}
