package com.app.security.auth

import grails.plugin.springsecurity.SpringSecurityService
import org.grails.datastore.mapping.engine.event.AbstractPersistenceEvent
import org.grails.datastore.mapping.engine.event.EventType
import org.grails.datastore.mapping.engine.event.PreInsertEvent
import org.grails.datastore.mapping.engine.event.PreUpdateEvent
import org.hibernate.event.spi.PreDeleteEvent
import org.springframework.beans.factory.annotation.Autowired
import grails.events.annotation.gorm.Listener
import groovy.transform.CompileStatic
import com.app.security.LoginUser

@CompileStatic
class LoginPasswordEncoderListener {

    @Autowired
    SpringSecurityService springSecurityService

    @Listener(LoginUser)
    void onPreInsertEvent(PreInsertEvent event) {
        encodeLoginUserPasswordForEvent(event)
    }

    @Listener(LoginUser)
    void onPreUpdateEvent(PreUpdateEvent event) {
        encodeLoginUserPasswordForEvent(event)
    }

    private void encodeLoginUserPasswordForEvent(AbstractPersistenceEvent event) {
        if (event.entityObject instanceof LoginUser) {
            LoginUser u = (event.entityObject as LoginUser)
            if (u.password && ((event instanceof  PreInsertEvent) || (event instanceof PreUpdateEvent && u.isDirty('password')))) {
                event.getEntityAccess().setProperty("password", encodeLoginUserPassword(u.password))
            }
        }
    }

    private String encodeLoginUserPassword(String password) {
        springSecurityService?.passwordEncoder ? springSecurityService.encodePassword(password) : password
    }
}